/**************************************************************************
*  This file is part of the TAL project (Tiny Abstraction Layer)
*
*  Copyright (c) 2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  23.02.2016  mifi  First Version, tested with an Embedded Artists 
*                    LPC1788 Developer's Kit.
**************************************************************************/
#if defined(USE_BOARD_EA1788)
#define __TALLED_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <stdint.h>
#include <string.h>
#include "tal.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*
 * LED status
 */
#define LED_OFF   0
#define LED_ON    1

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static uint8_t    LedStatus[TAL_LED_CHANNEL_MAX];
static uint16_t *pLEDAddr  = (uint16_t*)0x98000000;
static uint16_t  wLEDValue = 0;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  tal_LEDInit                                                          */
/*                                                                       */
/*  Initialize the LEDs of the EA-1788 board.                            */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void tal_LEDInit (void)
{
   /* Clear status information */
   memset(LedStatus, LED_OFF, sizeof(LedStatus));
   
   /* Init LED controlled by the memory bus */
   LPC_EMC->STATICCONFIG2   = 0x00000081;
   LPC_EMC->STATICWAITWEN2  = 0x00000003; /* ( n + 1 ) -> 4 clock cycles */
   LPC_EMC->STATICWAITOEN2  = 0x00000003; /* ( n )     -> 3 clock cycles */
   LPC_EMC->STATICWAITRD2   = 0x00000006; /* ( n + 1 ) -> 7 clock cycles */
   LPC_EMC->STATICWAITPAG2  = 0x00000003; /* ( n + 1 ) -> 4 clock cycles */
   LPC_EMC->STATICWAITWR2   = 0x00000005; /* ( n + 2 ) -> 7 clock cycles */
   LPC_EMC->STATICWAITTURN2 = 0x00000003; /* ( n + 1 ) -> 4 clock cycles */

   /* Clear all LEDs controlled by the memory bus */
   wLEDValue = 0;
   *pLEDAddr = wLEDValue;
   
   /* Turn on GPIO clock */
   Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_GPIO);
   
   /* Switch off all LEDs first to prevent glitches */   
   tal_LEDClear(TAL_LED_CHANNEL_1);
   tal_LEDClear(TAL_LED_CHANNEL_2);
   
   /* Set GPIO 2.26 and 2.27 to GPIO mode */
   Chip_IOCON_PinMuxSet(LPC_IOCON, 2, 26, IOCON_FUNC0);
   Chip_IOCON_PinMuxSet(LPC_IOCON, 2, 27, IOCON_FUNC0);
   
   /* Set GPIO 2.26 and 2.27 to output */
   Chip_GPIO_SetPinDIROutput(LPC_GPIO, 2, 26);
   Chip_GPIO_SetPinDIROutput(LPC_GPIO, 2, 27);
   
} /* tal_LEDInit */

/*************************************************************************/
/*  tal_LEDSet                                                           */
/*                                                                       */
/*  Set the given LED.                                                   */
/*                                                                       */
/*  In    : eChannel                                                     */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void tal_LEDSet (TAL_LED_CHANNEL eChannel)
{
   /* Check for valid range */
   if ((uint8_t)eChannel < (uint8_t)TAL_LED_CHANNEL_MAX)
   {
      switch (eChannel)
      {
         case TAL_LED_CHANNEL_1:  LPC_GPIO2->CLR = (1<<26); break; /* P2.26 */ 
         case TAL_LED_CHANNEL_2:  LPC_GPIO2->CLR = (1<<27); break; /* P2.27 */   
         
         case TAL_LED_CHANNEL_3:  wLEDValue |= (1<<0);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_4:  wLEDValue |= (1<<1);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_5:  wLEDValue |= (1<<2);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_6:  wLEDValue |= (1<<3);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_7:  wLEDValue |= (1<<4);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_8:  wLEDValue |= (1<<5);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_9:  wLEDValue |= (1<<6);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_10: wLEDValue |= (1<<7);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_11: wLEDValue |= (1<<8);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_12: wLEDValue |= (1<<9);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_13: wLEDValue |= (1<<10); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_14: wLEDValue |= (1<<11); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_15: wLEDValue |= (1<<12); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_16: wLEDValue |= (1<<13); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_17: wLEDValue |= (1<<14); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_18: wLEDValue |= (1<<15); *pLEDAddr = wLEDValue; break;
         default:
         {
            /* Do nothing */
            break;
         }
      }
      
      LedStatus[eChannel] = LED_ON;
   }
   
} /* hal_LEDSet */

/*************************************************************************/
/*  tal_LEDClear                                                         */
/*                                                                       */
/*  Clear the given LED.                                                 */
/*                                                                       */
/*  In    : eChannel                                                     */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void tal_LEDClear (TAL_LED_CHANNEL eChannel)
{
   /* Check for valid range */
   if ((uint8_t)eChannel < (uint8_t)TAL_LED_CHANNEL_MAX)
   {
      switch (eChannel)
      {
         case TAL_LED_CHANNEL_1: LPC_GPIO2->SET = (1<<26); break; /* P2.26 */ 
         case TAL_LED_CHANNEL_2: LPC_GPIO2->SET = (1<<27); break; /* P2.27 */   

         case TAL_LED_CHANNEL_3:  wLEDValue &= ~(1<<0);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_4:  wLEDValue &= ~(1<<1);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_5:  wLEDValue &= ~(1<<2);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_6:  wLEDValue &= ~(1<<3);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_7:  wLEDValue &= ~(1<<4);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_8:  wLEDValue &= ~(1<<5);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_9:  wLEDValue &= ~(1<<6);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_10: wLEDValue &= ~(1<<7);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_11: wLEDValue &= ~(1<<8);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_12: wLEDValue &= ~(1<<9);  *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_13: wLEDValue &= ~(1<<10); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_14: wLEDValue &= ~(1<<11); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_15: wLEDValue &= ~(1<<12); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_16: wLEDValue &= ~(1<<13); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_17: wLEDValue &= ~(1<<14); *pLEDAddr = wLEDValue; break;
         case TAL_LED_CHANNEL_18: wLEDValue &= ~(1<<15); *pLEDAddr = wLEDValue; break;
         
         default:
         {
            /* Do nothing */
            break;
         }
      }
      
      LedStatus[eChannel] = LED_OFF;
   }
   
} /* tal_LEDClear */

/*************************************************************************/
/*  tal_LEDToggle                                                        */
/*                                                                       */
/*  Toggle the given LED.                                                */
/*                                                                       */
/*  In    : eChannel                                                     */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void tal_LEDToggle (TAL_LED_CHANNEL eChannel)
{
   /* Check for valid range */
   if ((uint8_t)eChannel < (uint8_t)TAL_LED_CHANNEL_MAX)
   {
      if (LED_ON == LedStatus[eChannel])
      { 
         tal_LEDClear(eChannel); 
      }
      else
      {
         tal_LEDSet(eChannel); 
      }         
   }
   
} /* tal_LEDClear */

#endif /* USE_BOARD_EA1788 */

/*** EOF ***/
