/**************************************************************************
*  This file is part of the TAL project (Tiny Abstraction Layer)
*
*  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  The source is partial based on a LPCopen (v2.10) example.
*  Therefore partial Copyright(C) NXP Semiconductors, 2012
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  30.01.2017  mifi  First Version, tested with an Embedded Artists 
*                    LPC1788 Developer's Kit.
*  04.02.2017  mifi  Tested with an Embedded Artist
*                    LPC2468 Developer's Kit.
**************************************************************************/
#define __ENET_24XX_17XX_40XX_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <string.h>
#include <stdint.h>
#include "tal.h"

#include "enet_24xx_17xx_40xx.h"
#include "eth_phy_driver.h"

/*=======================================================================*/
/*  Extern                                                               */
/*=======================================================================*/

extern uint32_t FindMIIDiv (uint32_t ClockRate);

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define MULTICAST_FILTER_SIZE    32

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/* EMAC local DMA Descriptors */
static ENET_RXDESC_T RxDesc[ENET_NUM_BUFF_RXDESCS] __attribute__((section(".dma"),aligned(8)));
static ENET_RXSTAT_T RxStat[ENET_NUM_BUFF_RXDESCS] __attribute__((section(".dma"),aligned(8)));
static ENET_TXDESC_T TxDesc[ENET_NUM_BUFF_TXDESCS] __attribute__((section(".dma"),aligned(8)));
static ENET_TXSTAT_T TxStat[ENET_NUM_BUFF_TXDESCS] __attribute__((section(".dma"),aligned(8)));

/* EMAC local DMA buffers. */
static uint32_t RxBuf[ENET_NUM_BUFF_RXDESCS][ENET_ETH_MAX_FLEN>>2] __attribute__((section(".dma"),aligned(8)));
static uint32_t TxBuf[ENET_NUM_BUFF_TXDESCS][ENET_ETH_MAX_FLEN>>2] __attribute__((section(".dma"),aligned(8)));

static uint8_t  MyMACAddress[6];
static uint32_t MulticastFilterList[MULTICAST_FILTER_SIZE];

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*
 * ether_crc32_be based on FreeBSD:
 * https://lists.freebsd.org/pipermail/freebsd-current/2004-June/029754.html
 */
static uint32_t ether_crc32_be (const uint8_t *buf, size_t len)
{
   static const uint8_t rev[] = {
      0x0, 0x8, 0x4, 0xc, 0x2, 0xa, 0x6, 0xe,
      0x1, 0x9, 0x5, 0xd, 0x3, 0xb, 0x7, 0xf
   };
   static const uint32_t crctab[] = {
      0x00000000, 0x04c11db7, 0x09823b6e, 0x0d4326d9,
      0x130476dc, 0x17c56b6b, 0x1a864db2, 0x1e475005,
      0x2608edb8, 0x22c9f00f, 0x2f8ad6d6, 0x2b4bcb61,
      0x350c9b64, 0x31cd86d3, 0x3c8ea00a, 0x384fbdbd
   };
   size_t i;
   uint32_t crc;
   uint8_t data;

   crc = 0xffffffff; /* initial value */

   for (i = 0; i < len; i++) {
      data = buf[i];
      crc = (crc << 4) ^ crctab[(crc >> 28) ^ rev[data & 0xf]];
      crc = (crc << 4) ^ crctab[(crc >> 28) ^ rev[data >> 4]];
   }

   return (crc);
} /* ether_crc32_be */

/*************************************************************************/
/*  UpdateHashFilter                                                     */
/*                                                                       */
/*  Update the hash filter by the given address.                         */
/*                                                                       */
/*  In    : pAddress                                                     */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void UpdateHashFilter (uint8_t *pAddress)
{
   uint32_t dCRC;
   
   dCRC = ether_crc32_be(pAddress, 6);
   dCRC = (dCRC >> 23) & 0x3F;
   if (dCRC > 31)
   {
      dCRC = dCRC - 32;
      ENET_HASHFILTERH |= (1u << dCRC); 
   }
   else
   {
      ENET_HASHFILTERL |= (1u << dCRC);
   }
   
} /* UpdateHashFilter */

/*************************************************************************/
/*  InitDMARxDesc                                                        */
/*                                                                       */
/*  Initialize DMA RX descriptors.                                       */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void InitDMARxDesc (void)
{
   uint8_t  bIndex;
   
   /*
    * Setup RX Descriptors
    */
   for (bIndex = 0; bIndex < ENET_NUM_BUFF_RXDESCS; bIndex++)
   {
      RxDesc[bIndex].Packet  = (uint32_t)&RxBuf[bIndex][0]; 
      RxDesc[bIndex].Control = ENET_RCTRL_INT | (ENET_ETH_MAX_FLEN-1);
      
      RxStat[bIndex].StatusInfo    = 0;
      RxStat[bIndex].StatusHashCRC = 0;
   }

   /* Set EMAC Receive Descriptor Registers. */
   ENET_RX_DESCRIPTOR        = (uint32_t)RxDesc;
   ENET_RX_STATUS            = (uint32_t)RxStat;
   ENET_RX_DESCRIPTOR_NUMBER = ENET_NUM_BUFF_RXDESCS - 1;
                                      
   /* Rx Descriptors Point to 0 */
   ENET_RX_CONSUME_INDEX = 0;
   
} /* InitDMARxDesc */

/*************************************************************************/
/*  InitDMATxDesc                                                        */
/*                                                                       */
/*  Initialize DMA TX descriptors.                                       */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void InitDMATxDesc (void)
{
   uint8_t  bIndex;
   
   /*
    * Setup TX Descriptors
    */
   for (bIndex = 0; bIndex < ENET_NUM_BUFF_TXDESCS; bIndex++)
   {
      TxDesc[bIndex].Packet  = (uint32_t)&TxBuf[bIndex][0];
      TxDesc[bIndex].Control = 0;
      
      TxStat[bIndex].StatusInfo = 0;
   }      

   /* Set EMAC Transmit Descriptor Registers. */
   ENET_TX_DESCRIPTOR        = (uint32_t)TxDesc;;
   ENET_TX_STATUS            = (uint32_t)TxStat;;
   ENET_TX_DESCRIPTOR_NUMBER = ENET_NUM_BUFF_TXDESCS - 1;

   /* Tx Descriptors Point to 0 */
   ENET_TX_PRODUCE_INDEX = 0;
   
} /* InitDMATxDesc */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  enet_Init                                                            */
/*                                                                       */
/*  Initialize the ethernet controller.                                  */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_Init (uint8_t UseRMII)
{
   memset(RxBuf, 0x00, sizeof(RxBuf));
   memset(TxBuf, 0x00, sizeof(TxBuf));
   
   memset(MulticastFilterList, 0x00, sizeof(MulticastFilterList));
   
   /*
    * Clock and power are still enabled by GPIO_CONFIG
    */

   /* 
    * Put all modules within the MAC in reset
    */
   ENET_MAC1 = (ENET_MAC1_RESETTX    |  
                ENET_MAC1_RESETMCSTX |  
                ENET_MAC1_RESETRX    |  
                ENET_MAC1_RESETMCSRX |  
                ENET_MAC1_SIMRESET   |
                ENET_MAC1_SOFTRESET); 
         
   ENET_COMMAND = (ENET_COMMAND_REGRESET | 
                   ENET_COMMAND_TXRESET  |
                   ENET_COMMAND_RXRESET);

   /* A short delay after reset */
   OSTimeWait(2);
   
   /* Remove reset */
   ENET_MAC1    = 0;
   ENET_COMMAND = 0;

   /* 
    * Initial MAC configuration for  full duplex,
    * 100Mbps, inter-frame gap use default values.
    */
   ENET_MAC2 = ENET_MAC2_FULLDUPLEX | ENET_MAC2_CRCEN | ENET_MAC2_PADCRCEN;
   
   /* Select RMII or MII interface */
   if (UseRMII)
   {
      ENET_COMMAND = ENET_COMMAND_FULLDUPLEX | ENET_COMMAND_PASSRUNTFRAME | ENET_COMMAND_RMII;
   }
   else
   {
      ENET_COMMAND = ENET_COMMAND_FULLDUPLEX | ENET_COMMAND_PASSRUNTFRAME;
   }   
   
   ENET_IPGT = ENET_IPGT_FULLDUPLEX;
   ENET_IPGR = (ENET_IPGR_P1_DEF | ENET_IPGR_P2_DEF);
   ENET_SUPP = ENET_SUPP_100Mbps_SPEED;
   ENET_MAXF = ENET_MAXF_MAXFLEN_DEF;
   ENET_CLRT = ENET_CLRT_DEF;

   /* Setup the PHY link clock divider */
   ENET_MCFG = ENET_MCFG_RES_MII;
   OSTimeWait(2);
   ENET_MCFG = ENET_MCFG_CLOCKSEL(FindMIIDiv(2500000));
   
   /* Initialize DMA Descriptors */
   InitDMARxDesc();
   InitDMATxDesc();
   
   ENET_RX_FILTER_CTRL = (ENET_RXFILTERCTRL_ABE  |   /* All broadcast frames are accepted */
                          ENET_RXFILTERCTRL_AUHE |   /* Unicast frames that pass the imperfect hash filter are accepted */
                          ENET_RXFILTERCTRL_AMHE |   /* Multicast frames that pass the imperfect hash filter are accepted */
                          ENET_RXFILTERCTRL_APE);    /* The frames with a destination address identical to the station address are accepted */
                              
} /* enet_Init */

/*************************************************************************/
/*  enet_Start                                                           */
/*                                                                       */
/*  "Start" the ethernet controller.                                     */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_Start (void)
{
   /* Enable EMAC interrupts. */
   ENET_INT_ENABLE = (ENET_INT_RXOVERRUN  |
                      ENET_INT_RXERROR    |
                      ENET_INT_RXDONE     | 
                      ENET_INT_TXUNDERRUN |
                      ENET_INT_TXERROR    |
                      ENET_INT_TXDONE);    

   /* Reset all interrupts */
   ENET_INT_CLEAR = 0xFFFF;

   /* Enable receive and transmit mode of MAC Ethernet core */
   ENET_COMMAND |= (ENET_COMMAND_RXENABLE | ENET_COMMAND_TXENABLE);
   
   ENET_MAC1 |= ENET_MAC1_RXENABLE;  
   
} /* enet_Start */

/*************************************************************************/
/*  enet_GetIRQStatus                                                    */
/*                                                                       */
/*  Return the IRQ status.                                               */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: IRQStatus                                                    */
/*************************************************************************/
uint32_t enet_GetIRQStatus (void)
{
   return(ENET_INT_STATUS);
} /* enet_GetIRQStatus */

/*************************************************************************/
/*  enet_ClearIRQStatus                                                  */
/*                                                                       */
/*  Clear the IRQ status.                                                */
/*                                                                       */
/*  In    : dStatus                                                      */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_ClearIRQStatus (uint32_t dStatus)
{
   ENET_INT_CLEAR = dStatus;
   
} /* enet_ClearIRQStatus */

/*************************************************************************/
/*  enet_MACAddressConfig                                                */
/*                                                                       */
/*  Set the MAC address.                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_MACAddressConfig (uint8_t *pBuffer)
{
   /* Set the Ethernet MAC Address registers */
   ENET_SA0 = ((uint32_t) pBuffer[5] << 8) | ((uint32_t) pBuffer[4]);
   ENET_SA1 = ((uint32_t) pBuffer[3] << 8) | ((uint32_t) pBuffer[2]);
   ENET_SA2 = ((uint32_t) pBuffer[1] << 8) | ((uint32_t) pBuffer[0]);

   memcpy(MyMACAddress, pBuffer, sizeof(pBuffer));

   /* Update hash filter */   
   UpdateHashFilter(MyMACAddress);
   
} /* enet_MACAddressConfig */

/*************************************************************************/
/*  enet_MulticastAddFilter                                              */
/*                                                                       */
/*  Add a multicast address to the filter list.                          */
/*                                                                       */
/*  In    : dAddress                                                     */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_MulticastAddFilter (uint32_t dAddress)
{
   uint8_t Buffer[6];
   uint8_t bIndex;
   
   /*
    * Check if dAddress is still in list
    */
   for (bIndex = 0; bIndex < MULTICAST_FILTER_SIZE; bIndex++)
   {
      if (MulticastFilterList[bIndex] == dAddress)
      {
         return;
      }
   }    
   
   /*
    * Check for a free entry
    */
   for (bIndex = 0; bIndex < MULTICAST_FILTER_SIZE; bIndex++)
   {
      if (0 == MulticastFilterList[bIndex])
      {
         /* Save entry */
         MulticastFilterList[bIndex] = dAddress;
         
         /* Create MAC address */
         Buffer[0] = 0x01;
         Buffer[1] = 0x00;
         Buffer[2] = 0x5E;
         Buffer[3] = ((uint8_t *) &dAddress)[1] & 0x7f;
         Buffer[4] = ((uint8_t *) &dAddress)[2];
         Buffer[5] = ((uint8_t *) &dAddress)[3];

         UpdateHashFilter(Buffer);
         break;
      }
    }  

} /* enet_MulticastAddFilter */

/*************************************************************************/
/*  enet_MulticastDelFilter                                              */
/*                                                                       */
/*  Delete a multicast address from the filter and list.                 */
/*                                                                       */
/*  In    : dAddress                                                     */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_MulticastDelFilter (uint32_t dAddress)
{
   uint8_t Buffer[6];
   uint8_t bIndex;
   uint8_t bRebuildList = FALSE;
   
   /*
    * Check if dAddress is still in list
    */
   for (bIndex = 0; bIndex < MULTICAST_FILTER_SIZE; bIndex++)
   {
      if (MulticastFilterList[bIndex] == dAddress)
      {
         bRebuildList = TRUE;
         MulticastFilterList[bIndex] = 0;
         break;
      }
   }    

   if (TRUE == bRebuildList)
   {
      /* Clear hash filter */
      ENET_HASHFILTERH = 0;
      ENET_HASHFILTERL = 0;

      /* Add our station address first */
      UpdateHashFilter(MyMACAddress);
      
      /* Add multicast addresses from the list */
      for (bIndex = 0; bIndex < MULTICAST_FILTER_SIZE; bIndex++)
      {
         if (MulticastFilterList[bIndex] != 0)
         {
            dAddress = MulticastFilterList[bIndex];
            
            /* Create MAC address */
            Buffer[0] = 0x01;
            Buffer[1] = 0x00;
            Buffer[2] = 0x5E;
            Buffer[3] = ((uint8_t *) &dAddress)[1] & 0x7f;
            Buffer[4] = ((uint8_t *) &dAddress)[2];
            Buffer[5] = ((uint8_t *) &dAddress)[3];

            UpdateHashFilter(Buffer);
         }
      }
   }

} /* enet_MulticastDelFilter */

/*************************************************************************/
/*  enet_UpdateLink                                                      */
/*                                                                       */
/*  Update the link status inside the ethernet controller.               */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_UpdateLink (uint8_t bLinkStatus)
{
   uint32_t dSUPP;
   uint32_t dMAC2;
   uint32_t dIPGT;
   uint32_t dCommand;
   
   dSUPP    = (ENET_SUPP    & ~ENET_SUPP_100Mbps_SPEED);
   dMAC2    = (ENET_MAC2    & ~ENET_MAC2_FULLDUPLEX);
   dCommand = (ENET_COMMAND & ~ENET_COMMAND_FULLDUPLEX);
   
   /* Configure Full/Half Duplex mode */
   if (bLinkStatus & PHY_LINK_STATUS_MODE_FULL)
   {
      dMAC2    |= ENET_MAC2_FULLDUPLEX;
      dIPGT     = ENET_IPGT_FULLDUPLEX;
      dCommand |= ENET_COMMAND_FULLDUPLEX;
   }
   else
   {
      dIPGT = ENET_IPGT_HALFDUPLEX;
   }

   /* Configure 100MBit/10MBit mode. */
   if (bLinkStatus & PHY_LINK_STATUS_SPEED_100M)
   {
      dSUPP |= ENET_SUPP_100Mbps_SPEED;
   }
   
   ENET_SUPP    = dSUPP;
   ENET_MAC2    = dMAC2;
   ENET_IPGT    = dIPGT;
   ENET_COMMAND = dCommand;
   
} /* enet_UpdateLink */

/*************************************************************************/
/*  enet_PHYRegRead                                                      */
/*                                                                       */
/*  Read a PHY register and return the value.                            */
/*  (See UM10237, Rev. 04 - 26 August 2009, page 227)                    */
/*                                                                       */
/*  In    : wAddress, wReg                                               */
/*  Out   : none                                                         */
/*  Return: wValue                                                       */
/*************************************************************************/
uint16_t enet_PHYRegRead (uint16_t wAddress, uint16_t wReg)
{
   uint16_t wValue = 0;
   uint16_t wMADR;
   
   if (wAddress != 0)
   {
      wAddress &= 0x1F;
      wReg     &= 0x1F;
   
      wMADR = (uint16_t)((wAddress << 8) | wReg);

      ENET_MCMD = ENET_MCMD_READ;
      ENET_MADR = wMADR;
      
      /* Wait for busy bit to be cleared */
      while (ENET_MIND & ENET_MIND_BUSY)
      {
         __asm volatile ("nop");
      }
      
      ENET_MCMD = 0;

      wValue = (uint16_t)(ENET_MRDD & 0xFFFF);
   }      
   
   return(wValue);
} /* enet_PHYRegRead */

/*************************************************************************/
/*  enet_PHYRegWrite                                                     */
/*                                                                       */
/*  Write a PHY register.                                                */
/*  (See UM10237, Rev. 04 - 26 August 2009, page 227)                    */
/*                                                                       */
/*  In    : wAddress, wReg                                               */
/*  Out   : none                                                         */
/*  Return: wValue                                                       */
/*************************************************************************/
void enet_PHYRegWrite (uint16_t wAddress, uint16_t wReg, uint16_t wValue)
{
   uint16_t wMADR;
   
   if (wAddress != 0)
   {
      wAddress &= 0x1F;
      wReg     &= 0x1F;
   
      wMADR = (uint16_t)((wAddress << 8) | wReg);

      ENET_MCMD = 0;
      ENET_MADR = wMADR;
      ENET_MWTD = wValue;

      /* Wait for busy bit to be cleared */
      while (ENET_MIND & ENET_MIND_BUSY)
      {
         __asm volatile ("nop");
      }
   }      
} /* enet_PHYRegWrite */

/*************************************************************************/
/*  enet_RxFrameGet                                                      */
/*                                                                       */
/*  Return the buffer and size if a RX frame is available.               */
/*                                                                       */
/*  In    : pSize                                                        */
/*  Out   : pSize                                                        */
/*  Return: pBuffer / NULL                                               */
/*************************************************************************/
uint8_t *enet_RxFrameGet (uint16_t *pSize)
{
   uint8_t *pBuffer = NULL;
   uint16_t wIndex;
   uint16_t wSize;
   
   if (ENET_RX_PRODUCE_INDEX != ENET_RX_CONSUME_INDEX)
   {
      wIndex  = (ENET_RX_CONSUME_INDEX & 0xFFFF);
      wSize   = (RxStat[wIndex].StatusInfo & 0x07FF) - 3;
      pBuffer = (uint8_t*)RxDesc[wIndex].Packet;
      
      *pSize = wSize;
   }
   
   return(pBuffer);
} /* enet_RxFrameGet */

/*************************************************************************/
/*  enet_RxFrameReady                                                    */
/*                                                                       */
/*  Free the RX frame.                                                   */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_RxFrameReady (void)
{
   uint16_t wIndex;

   /* Get current RX consume index */
   wIndex = (ENET_RX_CONSUME_INDEX & 0xFFFF);
   
   wIndex++;
   if (ENET_NUM_BUFF_RXDESCS == wIndex)
   {
      wIndex = 0;
   }
   ENET_RX_CONSUME_INDEX = wIndex;

} /* enet_RxFrameReady */

/*************************************************************************/
/*  enet_RxOverrun                                                       */
/*                                                                       */
/*  Handle RX overrun.                                                   */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_RxOverrun (void)
{
   /* Temporarily disable RX */
	ENET_COMMAND &= ~ENET_COMMAND_RXENABLE;
	ENET_MAC1    &= ~ENET_MAC1_RXENABLE;

   /* Reset the RX side */
	ENET_MAC1 |=  ENET_MAC1_RESETRX;
   ENET_MAC1 &= ~ENET_MAC1_RESETRX;

   InitDMARxDesc();
   
   /* Re-enable RX */
   ENET_COMMAND |= ENET_COMMAND_RXENABLE;
	ENET_MAC1    |= ENET_MAC1_RXENABLE;

} /* enet_RxOverrun */

/*************************************************************************/
/*  enet_TxFrameGet                                                      */
/*                                                                       */
/*  Return the buffer for a TX frame if available.                       */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: pBuffer                                                      */
/*************************************************************************/
uint8_t *enet_TxFrameGet (void)
{
   uint8_t *pBuffer;
   uint16_t wIndex;
   
   wIndex  = (ENET_TX_PRODUCE_INDEX & 0xFFFF);
   pBuffer = (uint8_t*)TxDesc[wIndex].Packet;
   
   return(pBuffer);
} /* enet_TxFrameGet */

/*************************************************************************/
/*  enet_TxFrameSend                                                     */
/*                                                                       */
/*  Send the TX frame.                                                   */
/*                                                                       */
/*  In    : wSize                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_TxFrameSend (uint16_t wSize)
{
   uint16_t wIndex;
   
   /* Pad frame if it is less than 60 */ 
   wSize = (wSize < 60) ? 60 : wSize;

   /* Get current TX produce index */
   wIndex = (ENET_TX_PRODUCE_INDEX & 0xFFFF);
   TxDesc[wIndex].Control = (wSize -1)           | 
                             ENET_TCTRL_OVERRIDE |
                             ENET_TCTRL_PAD      |
                             ENET_TCTRL_CRC      |
                             ENET_TCTRL_LAST     |
                             ENET_TCTRL_INT;     

   /* Start frame transmission by incrementing the descriptor */
   wIndex++;
   if (ENET_NUM_BUFF_TXDESCS == wIndex)
   {
      wIndex = 0;
   }
   ENET_TX_PRODUCE_INDEX = wIndex;

} /* enet_TxFrameSend */

/*************************************************************************/
/*  enet_TxUnderrun                                                      */
/*                                                                       */
/*  Handle TX underrun.                                                  */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void enet_TxUnderrun (void)
{
   /* Temporarily disable TX */
	ENET_COMMAND &= ~ENET_COMMAND_TXENABLE;

   /* Reset the TX side */
	ENET_MAC1 |=  ENET_MAC1_RESETTX;
   ENET_MAC1 &= ~ENET_MAC1_RESETTX;

   InitDMATxDesc();
   
   /* Re-enable TX */
   ENET_COMMAND |= ENET_COMMAND_TXENABLE;

} /* enet_TxUnderrun */

/*** EOF ***/
