/**************************************************************************
*  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  23.08.2017  mifi  First Version.
**************************************************************************/
#define __UPD_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include "tal.h"
#include "upd.h"
#include "crc16.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define READY_TIMEOUT_TICKS   OS_MS_2_TICKS(500)


/*
 * Register definition
 */
typedef struct
{
  volatile uint8_t DATA;
  volatile uint8_t FRXC;
  volatile uint8_t FTXC;
  volatile uint8_t LSR;
  volatile uint8_t RES4;
  volatile uint8_t ID5;
  volatile uint8_t ID6;
  volatile uint8_t INFO;
} UART_TypeDef;


#define UART_REG_DATA   0
#define UART_REG_FRXC   1
#define UART_REG_FTXC   2
#define UART_REG_LSR    3
#define UART_REG_RES4   4
#define UART_REG_ID5    5
#define UART_REG_ID6    6
#define UART_REG_INFO   7

#define UART_FIFO_SIZE  256

#define LSR_DR          (1<<0)
#define LSR_THRE        (1<<5)
#define LSR_READY       (1<<6)

#define INFO_READY      (1<<7)


#define CMD_BUFFER_SIZE          64

#define CMD_VERSION              "<v>"
#define CMD_IMAGE                "<i>"
#define CMD_START                "<s>"
#define CMD_ERASE                "<e>"
#define CMD_ERASE_ALL            "<E>"
#define CMD_XMODEM               "<x>"

#define CMD_VERSION_TIMEOUT      OS_MS_2_TICKS(50)
#define CMD_IMAGE_TIMEOUT        OS_MS_2_TICKS(50)
#define CMD_START_TIMEOUT        OS_MS_2_TICKS(50)
#define CMD_ERASE_TIMEOUT        OS_MS_2_TICKS(50)
#define CMD_XMODEM_TIMEOUT       OS_MS_2_TICKS(50)
#define TARGET_RES_TIMEOUT       OS_MS_2_TICKS(1000)

#define XMODEM_MAX_NAK_COUNTER   10
#define XMODEM_MAX_FRAME_SIZE    128
#define FRAME_BUFFER_SIZE        (1+2+XMODEM_MAX_FRAME_SIZE+2)

#define SOH    0x01  /* Start of frame 128 */
#define EOT    0x04
#define ACK    0x06
#define NAK    0x15
#define CAN    0x18
#define EOF    0x1a

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static uint8_t CMDBuffer[CMD_BUFFER_SIZE];

static uint8_t FrameBuffer[FRAME_BUFFER_SIZE];

static uint32_t dUpdateCountMax = 0;
static uint32_t dUpdateCount    = 0;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  UARTRegRead                                                          */
/*                                                                       */
/*  Return the value of the given UART register.                         */
/*                                                                       */
/*  In    : bReg                                                         */
/*  Out   : none                                                         */
/*  Return: bValue                                                       */
/*************************************************************************/
static uint8_t UARTRegRead (uint8_t bReg)
{
   uint8_t      bValue = 0;
   UART_TypeDef* UART  = (UART_TypeDef*)0x9C000000;
   
   switch (bReg)
   {
      case UART_REG_DATA: bValue = (uint8_t)UART->DATA; break;
      case UART_REG_FRXC: bValue = (uint8_t)UART->FRXC; break;
      case UART_REG_FTXC: bValue = (uint8_t)UART->FTXC; break;
      case UART_REG_LSR : bValue = (uint8_t)UART->LSR;  break;
      case UART_REG_ID5:  bValue = (uint8_t)UART->ID5;  break;
      case UART_REG_ID6:  bValue = (uint8_t)UART->ID6;  break;
      case UART_REG_INFO: bValue = (uint8_t)UART->INFO; break;
           
      default:
      {
         /* Do nothing */
         break;
      }
   }
   
   return(bValue);
} /* UARTRegRead */

/*************************************************************************/
/*  UARTRegWrite                                                         */
/*                                                                       */
/*  Write the value to the given UART register.                          */
/*                                                                       */
/*  In    : bReg, bValue                                                 */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void UARTRegWrite (uint8_t bReg, uint8_t bValue)
{
   UART_TypeDef* UART = (UART_TypeDef*)0x9C000000;

   switch (bReg)
   {
      case UART_REG_DATA: UART->DATA = bValue; break;
      case UART_REG_FRXC: UART->FRXC = bValue; break;
      case UART_REG_FTXC: UART->FTXC = bValue; break;
      
      default:
      {
         /* Do nothing */
         break;
      }
   }
   
} /* UARTRegWrite */

/*************************************************************************/
/*  COMSendBlock                                                         */
/*                                                                       */
/*  Send a block of data.                                                */
/*                                                                       */
/*  In    : pBuffer, wSize                                               */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void COMSendBlock (uint8_t *pBuffer, uint16_t wSize)
{
   uint8_t bReg;
   
   while (wSize)
   {
      bReg = UARTRegRead(UART_REG_LSR);
      if (LSR_THRE == (bReg & LSR_THRE))
      {
         UARTRegWrite(UART_REG_DATA, *pBuffer++);
         wSize--;
      }
   }
} /* COMSendBlock */

/*************************************************************************/
/*  COMReceiveData                                                       */
/*                                                                       */
/*  Get one data if available.                                           */
/*                                                                       */
/*  In    : pData                                                        */
/*  Out   : pData                                                        */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
static TAL_RESULT COMReceiveData (uint8_t *pData)
{
   TAL_RESULT Result = TAL_ERROR;
   uint8_t   bReg;
   
   bReg = UARTRegRead(UART_REG_LSR);
   if (LSR_DR == (bReg & LSR_DR))
   {
      Result = TAL_OK;
      *pData = UARTRegRead(UART_REG_DATA);
   }
   
   return(Result);
} /* COMReceiveData */

/*************************************************************************/
/*  CheckDeviceReady                                                     */
/*                                                                       */
/*  Check if the device is "Ready".                                      */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static TAL_RESULT CheckDeviceReady (void)
{
   TAL_RESULT Result = TAL_ERROR;
   uint8_t   bReg;
   uint8_t   bReady;
   uint32_t  dEndTime;
   
   bReady = FALSE;
   dEndTime = OSTimeGet() + READY_TIMEOUT_TICKS;
   
   while (OSTimeGet() < dEndTime)
   {
      bReg = UARTRegRead(UART_REG_INFO);
      if (INFO_READY == (bReg & INFO_READY))
      {
         bReady = TRUE;
         break;
      }
   }
   
   if (TRUE == bReady)
   {
      Result = TAL_OK;
   }
   
   return(Result);
} /* CheckDeviceReady */

/*************************************************************************/
/*  GetImageTimeName                                                     */
/*                                                                       */
/*  Decode the CMDBuffer and get the image time and name.                */
/*                                                                       */
/*  In    : pTime, pName                                                 */
/*  Out   : pTime, pName                                                 */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
static TAL_RESULT GetImageTimeName (uint32_t *pTime, char *pName)
{
   TAL_RESULT Result = TAL_ERROR;
   char      *pStart;
   char      *pEnd;
   long long int Time;

   *pTime = 0;
   *pName = 0;
   
   /* The CMDBuffer looks like "<i:time:"name">" */
   
   if (0 == strncmp((char*)CMDBuffer, "<i:", 3))
   {
      /* Get Time */
      pStart = (char*)&CMDBuffer[3];
      pEnd   = strstr(pStart, ":");
      if (pEnd != NULL)
      {
         *pEnd  = 0;
         Time   = atoll(pStart);
         *pTime = (uint32_t)Time;
         
         /* Get Name */
         pStart = pEnd;
         pStart++;
         pEnd = strstr(pStart, ">");
         if (pEnd != NULL)
         {
            *pEnd = 0;
            memcpy(pName, pStart, strlen(pStart) + 1);
            
            Result = TAL_OK;
         }
      }
   }

   return(Result);
} /* GetImageTimeName */

/*************************************************************************/
/*  GetBootloaderVersion                                                 */
/*                                                                       */
/*  Decode the CMDBuffer and get the bootloader version.                 */
/*                                                                       */
/*  In    : pVersion1, pVersion2                                         */
/*  Out   : pVersion1, pVersion2                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
static TAL_RESULT GetBootloaderVersion (uint16_t *pVersion1, uint16_t *pVersion2)
{
   TAL_RESULT Result = TAL_ERROR;
   char      *pStart;
   char      *pEnd;

   /* The CMDBuffer looks like "<v:version1:version2>" */
   
   if (0 == strncmp((char*)CMDBuffer, "<v:", 3))
   {
      /* Get version1 */
      pStart = (char*)&CMDBuffer[3];
      pEnd   = strstr(pStart, ":");
      if (pEnd != NULL)
      {
         *pEnd = 0;
         *pVersion1 = (uint16_t)atoi(pStart);
         
         /* Get version2 */
         pStart = pEnd;
         pStart++;
         pEnd = strstr(pStart, ">");
         if (pEnd != NULL)
         {
            *pEnd = 0;            
            *pVersion2 = (uint16_t)atoi(pStart);
            
            Result = TAL_OK;
         }
      }
   }

   return(Result);
} /* CheckCMDResponse */

/*************************************************************************/
/*  CheckCMD                                                             */
/*                                                                       */
/*  Check the command response.                                          */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
static TAL_RESULT CheckCMD (char *pCMD)
{
   TAL_RESULT Result = TAL_ERROR;
   
   /* The CMDBuffer looks like "<s>" */
   
   if (0 == strcmp((char*)CMDBuffer, pCMD))
   {
      Result = TAL_OK;
   }

   return(Result);
} /* CheckCMD */

/*************************************************************************/
/*  GetCMDResponse                                                       */
/*                                                                       */
/*  Get command response, with timeout.                                  */
/*                                                                       */
/*  In    : dTimeout                                                     */
/*  Out   : none                                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
static TAL_RESULT GetCMDResponse (uint32_t dTimeout)
{
   TAL_RESULT Result = TAL_ERROR;
   uint16_t  wSize   = (CMD_BUFFER_SIZE - 1);
   uint32_t  dStartTime;
   uint32_t  dEndTime = 0;
   uint32_t  dMaxTime;
   uint16_t  wIndex;
   uint8_t   bData;

   dStartTime = OSTimeGet();
   dMaxTime = dStartTime + dTimeout;
   
   wIndex = 0;
   while ((wIndex < wSize) && (OSTimeGet() < dMaxTime))
   {
      if (TAL_OK == COMReceiveData(&bData))
      {
         CMDBuffer[wIndex++] = bData;
         
         if ('>' == bData)
         {
            dEndTime = OSTimeGet();
            CMDBuffer[wIndex] = 0;
            Result = TAL_OK;
            break;
         } 
      }
   }
   
   (void)dEndTime;
   
   return(Result);
} /* GetCMDResponse */

/*************************************************************************/
/*  SendCMD                                                              */
/*                                                                       */
/*  Send command over the UART.                                          */
/*                                                                       */
/*  In    : pCMD                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SendCMD (char *pCMD)
{
   /* Clear RX-FIFO first */
   UARTRegWrite(UART_REG_FRXC, 0x00);
    
   COMSendBlock((uint8_t*)pCMD, (uint16_t)strlen(pCMD));   
} /* SendCMD */


/*************************************************************************/
/*  WaitXModemResponse                                                   */
/*                                                                       */
/*  Wait for XModem response, with timeout.                              */
/*                                                                       */
/*  In    : pData, dTimeout                                              */
/*  Out   : none                                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
static TAL_RESULT WaitXModemResponse (uint8_t *pData, uint32_t dTimeout)
{
   TAL_RESULT Result = TAL_ERROR;
   uint32_t  dMaxTime;
   uint8_t   bData;

   dMaxTime = OSTimeGet() + dTimeout;
   
   while (OSTimeGet() < dMaxTime)
   {
      if (TAL_OK == COMReceiveData(&bData))
      {
         *pData = bData;
         Result = TAL_OK;
         break;
      }
   }
   
   return(Result);
} /* WaitXModemResponse */

/*************************************************************************/
/*  XModemSend                                                           */
/*                                                                       */
/*  Send the image to the bootloader.                                    */
/*                                                                       */
/*  In    : pImage, dImageSize                                           */
/*  Out   : none                                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
static TAL_RESULT XModemSend (uint8_t *pImage, uint32_t dImageSize)
{
   TAL_RESULT Result = TAL_ERROR;
   uint8_t   bData;
   uint8_t   bRXPacketNumber = 1;
   uint16_t  wFrameSize;
   uint16_t  wFrameBufferSize;
   uint16_t  wCRC;
   uint8_t   bNAKCounter = 0;
   
   dUpdateCount = 0;
   
   while (1)
   {
      Result = COMReceiveData(&bData);
      if (TAL_OK == Result)
      {
         if ('C' == bData)
         {
            break;
         }
      }
   }
   
   /* Clear RX-FIFO first */
   UARTRegWrite(UART_REG_FRXC, 0x00);
   
   wFrameSize       = XMODEM_MAX_FRAME_SIZE;
   wFrameBufferSize = FRAME_BUFFER_SIZE;
   
   while (dImageSize > 0)
   {
      /* Build frame */
      FrameBuffer[0] = SOH;
      FrameBuffer[1] = bRXPacketNumber;
      FrameBuffer[2] = 0xFF - bRXPacketNumber;
      memcpy(&FrameBuffer[3], pImage, wFrameSize);
      wCRC = crc16_ccitt(0, &FrameBuffer[3], wFrameSize);
      FrameBuffer[(wFrameBufferSize-2)] = (uint8_t)((wCRC >> 8) & 0x00FF);
      FrameBuffer[(wFrameBufferSize-1)] = (uint8_t)(wCRC & 0x00FF);
      
      /* Send frame */
      COMSendBlock(FrameBuffer, wFrameBufferSize);
      
      /* Wait for response */
      Result = WaitXModemResponse(&bData, TARGET_RES_TIMEOUT);
      if (TAL_ERROR == Result)
      {
         /* Timeout */   
         FrameBuffer[0] = CAN;    
         FrameBuffer[1] = CAN;    
         FrameBuffer[2] = CAN;    
         FrameBuffer[3] = CAN;    
         COMSendBlock(FrameBuffer, 4);
         break;
      }
      else
      {
         switch(bData)
         {
            case ACK:
            {
               dImageSize -= wFrameSize;
               pImage     += wFrameSize;
               
               bRXPacketNumber++;
               dUpdateCount++;
               bNAKCounter = 0;
               break;
            }
            
            case NAK:
            {
               bNAKCounter++;
               break;
            }
            
            case CAN:
            {
               /* Abort by bootloader */
               goto XModemSendEnd; /*lint !e801*/
               break;              /*lint !e527*/
            }
            
            default:
            {
               bNAKCounter++;
               break;
            }
         }
      }
      
      /* Check if transfer must be aborted */
      if (bNAKCounter > XMODEM_MAX_NAK_COUNTER)
      {
         FrameBuffer[0] = CAN;    
         FrameBuffer[1] = CAN;    
         FrameBuffer[2] = CAN;    
         FrameBuffer[3] = CAN;    
         COMSendBlock(FrameBuffer, 4);
         break;
      }
      
      OSTimeDly(2);
   } /* end while (dImageSize > 0) */ 


XModemSendEnd: 

   Result = TAL_ERROR;
   
   /* Check if complete image was send */
   if (0 == dImageSize)
   {
      FrameBuffer[0] = EOT;    
      FrameBuffer[1] = EOT;    
      FrameBuffer[2] = EOT;    
      FrameBuffer[3] = EOT;    
      COMSendBlock(FrameBuffer, 4);
      
      Result = TAL_OK;
   }   
  
   return(Result);
} /* XModemSend */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  upd_Init                                                             */
/*                                                                       */
/*  Initialize the UPD functionality.                                    */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */ 
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
void upd_Init (void)
{
   LPC_EMC->STATICCONFIG3   = 0x00000080;
   LPC_EMC->STATICWAITWEN3  = 0x00000000;
   LPC_EMC->STATICWAITOEN3  = 0x00000000;
   LPC_EMC->STATICWAITRD3   = 0x00000004;
   LPC_EMC->STATICWAITPAG3  = 0x00000003;
   LPC_EMC->STATICWAITWR3   = 0x00000008;
   LPC_EMC->STATICWAITTURN3 = 0x00000003;

} /* upd_Init */

/*************************************************************************/
/*  upd_FPGACheck                                                        */
/*                                                                       */
/*  Check if the FPGA bootloader is active.                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */ 
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
TAL_RESULT upd_FPGACheckBLActive (void)
{
   TAL_RESULT  Result;

   Result = CheckDeviceReady();
   if (TAL_OK == Result)
   {
      /* Check if the bootloader is active */
      if (('B' == UARTRegRead(UART_REG_ID5)) && 
          ('O' == UARTRegRead(UART_REG_ID6)) )
      {
         Result = TAL_OK;
      }          
   }

   return(Result);
} /* upd_FPGACheckBLActive */

/*************************************************************************/
/*  upd_FPGAGetVersion                                                   */
/*                                                                       */
/*  Return the FPGA bootloader version.                                  */
/*                                                                       */
/*  In    : pVersion1, pVersion2                                         */
/*  Out   : pVersion1, pVersion2                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
TAL_RESULT upd_FPGAGetVersion (uint16_t *pVersion1, uint16_t *pVersion2)
{
   TAL_RESULT  Result;
   
   *pVersion1 = 0;
   *pVersion2 = 0;
   
   Result = upd_FPGACheckBLActive();
   if (TAL_OK == Result)
   {
      SendCMD(CMD_VERSION);
      Result = GetCMDResponse(CMD_VERSION_TIMEOUT);
      if (TAL_OK == Result)
      {
         Result = GetBootloaderVersion(pVersion1, pVersion2);
      }
   }      
   
   return(Result);   
} /* upd_FPGAGetVersion */

/*************************************************************************/
/*  upd_FPGAGetImageInfo                                                 */
/*                                                                       */
/*  Return the creation time and name of the FPGA image.                 */
/*                                                                       */
/*  In    : pTime, pName                                                 */
/*  Out   : pTime, pName                                                 */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
TAL_RESULT upd_FPGAGetImageInfo (uint32_t *pTime, char *pName)
{
   TAL_RESULT Result;
   
   *pTime = 0;
   *pName = 0;

   Result = upd_FPGACheckBLActive();
   if (TAL_OK == Result)
   {
      SendCMD(CMD_IMAGE);
      Result = GetCMDResponse(CMD_IMAGE_TIMEOUT);
      if (TAL_OK == Result)
      {
         Result = GetImageTimeName(pTime, pName);
      }
   }      
   
   return(Result);
} /* upd_FPGAGetImageInfo */

/*************************************************************************/
/*  upd_FPGAImageStart                                                   */
/*                                                                       */
/*  Start the FPGA image if available.                                   */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
TAL_RESULT upd_FPGAImageStart (void)
{
   TAL_RESULT Result;
   
   Result = upd_FPGACheckBLActive();
   if (TAL_OK == Result)
   {
      SendCMD(CMD_START);
      Result = GetCMDResponse(CMD_START_TIMEOUT);
      if (TAL_OK == Result)
      {
         Result = CheckCMD(CMD_START);
      }
   }      
   
   return(Result);
} /* upd_FPGAImageStart */

/*************************************************************************/
/*  upd_FPGAImageErase                                                   */
/*                                                                       */
/*  Erase the FPGA image if available.                                   */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
TAL_RESULT upd_FPGAImageErase (void)
{
   TAL_RESULT Result;
   
   Result = upd_FPGACheckBLActive();
   if (TAL_OK == Result)
   {
      SendCMD(CMD_ERASE);
      Result = GetCMDResponse(CMD_ERASE_TIMEOUT);
      if (TAL_OK == Result)
      {
         Result = CheckCMD(CMD_ERASE);
      }
   }      
   
   return(Result);
} /* upd_FPGAImageErase */

/*************************************************************************/
/*  upd_FPGAEraseAll                                                     */
/*                                                                       */
/*  Erase the FPGA uaser area.                                           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
TAL_RESULT upd_FPGAEraseAll (void)
{
   TAL_RESULT Result;
   
   Result = upd_FPGACheckBLActive();
   if (TAL_OK == Result)
   {
      SendCMD(CMD_ERASE_ALL);
      Result = GetCMDResponse(CMD_ERASE_TIMEOUT);
      if (TAL_OK == Result)
      {
         Result = CheckCMD(CMD_ERASE_ALL);
      }
   }      
   
   return(Result);
} /* upd_FPGAEraseAll */

/*************************************************************************/
/*  upd_FPGAStartXModem                                                  */
/*                                                                       */
/*  Start the FPGA XModem process.                                       */
/*                                                                       */
/*  In    : pImage, dImageSize                                           */
/*  Out   : none                                                         */
/*  Return: TAL_OK / TAL_ERROR                                           */
/*************************************************************************/
TAL_RESULT upd_FPGAStartXModem (uint8_t *pImage, uint32_t dImageSize)
{
   TAL_RESULT Result;

   Result = upd_FPGACheckBLActive();
   if (TAL_OK == Result)
   {
      SendCMD(CMD_XMODEM);
      Result = GetCMDResponse(CMD_XMODEM_TIMEOUT);
      if (TAL_OK == Result)
      {
         Result = CheckCMD(CMD_XMODEM);
      }
   }
   
   /* Check if the bootloader XModem process is started */
   if (TAL_OK == Result)
   {
      dUpdateCountMax = dImageSize / XMODEM_MAX_FRAME_SIZE;
      dUpdateCount    = 0;
   
      Result = XModemSend(pImage, dImageSize);
      
      dUpdateCountMax = 0;
      dUpdateCount    = 0;
   }      

   return(Result);
} /* upd_FPGAStartXModem */

/*************************************************************************/
/*  upd_FPGAGetInfo                                                      */
/*                                                                       */
/*  Return the FPGA bootloader info.                                     */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Info                                                         */
/*************************************************************************/
uint8_t upd_FPGAGetInfo (void)
{
   return(UARTRegRead(UART_REG_INFO));
} /* upd_FPGAGetInfo */

/*************************************************************************/
/*  upd_FPGAGetProgress                                                  */
/*                                                                       */
/*  Return the FPGA XModem transfer progress.                            */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Progress in %                                                */
/*************************************************************************/
uint8_t upd_FPGAGetProgress (void)
{
   uint32_t dValue = 0;
   
   if (dUpdateCountMax != 0)
   {
      dValue = (dUpdateCount * 100) / dUpdateCountMax;
   }
   else
   {
      dValue = 100;
   }

   return((uint8_t)dValue);
} /* upd_FPGAGetProgress */

/*** EOF ***/


